DROP TYPE COMP_PROD_ORE_TABLE
/

DROP TYPE COMP_PROD_ORE_TYPE
/

CREATE TYPE COMP_PROD_ORE_TYPE AS
OBJECT( CDPRODUTO     VARCHAR2(10),
        CDSUBPRODUTO  VARCHAR2(10),	
        NRNIVEL       NUMBER, 
        QTSUBPRODUTO  FLOAT,
        QTSUBPRODCALC FLOAT,
        IDVERIFCOMP   VARCHAR2(1),
        NRORDEM       NUMBER,
        SGUNIDADE     VARCHAR2(10), 	
        NMPRODUTO     VARCHAR2(50),
        NMFANTPRODUTO VARCHAR2(250))
/

CREATE TYPE COMP_PROD_ORE_TABLE AS TABLE OF COMP_PROD_ORE_TYPE
/

CREATE OR REPLACE FUNCTION COMP_PROD_ORE (P_CDFILIAL  IN VARCHAR2,
                                          P_CDPRODUTO IN VARCHAR2,
                                          P_QUANTIDADE IN FLOAT,
                                          P_NRNIVEL IN INTEGER DEFAULT 1,
                                          P_ORIGEM IN VARCHAR2 DEFAULT '') RETURN COMP_PROD_ORE_TABLE PIPELINED
IS
   V_COTLOCAL DOUBLE PRECISION;
   V_COTPADR DOUBLE PRECISION;
   V_IDTPPRODPEDOREV VARCHAR2(1);
   OUT_RECORD COMP_PROD_ORE_TYPE := COMP_PROD_ORE_TYPE (NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
BEGIN

   BEGIN
      SELECT IDTPPRODPEDOREV
        INTO V_IDTPPRODPEDOREV
        FROM PARAMORCAEVE;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
          V_IDTPPRODPEDOREV := 'O';
   END;

   SELECT COUNT(*)
     INTO V_COTLOCAL
     FROM ROTFABLOCAL R
    WHERE (R.CDFILIAL = P_CDFILIAL)
      AND (R.CDPRODUTO = P_CDPRODUTO);
   IF V_COTLOCAL > 0 THEN
      /* Composio local */
      FOR C_COMPLOCAL IN (SELECT R.CDPRODUTO, R.CDSUBPRODLOC, R.QTSUBPRODLOC, R.NRSEQPRODLOC, P.SGUNIDADE, P.NMPRODUTO, P.NMFANTPRODUTO
                            FROM ROTFABLOCAL R, PRODUTO P
                           WHERE (R.CDFILIAL = P_CDFILIAL)
                             AND (R.CDPRODUTO = P_CDPRODUTO)
                             AND (P.CDPRODUTO = R.CDSUBPRODLOC)
                             AND (NVL(R.IDLOCGERAPEDORCAEVE,'S') = CASE WHEN V_IDTPPRODPEDOREV = 'A' AND P_ORIGEM = 'PRODECOMP' 
                                                                        THEN 'S' 
                                                                        ELSE NVL(R.IDLOCGERAPEDORCAEVE, 'S') END)
                           ORDER BY R.NRSEQPRODLOC)LOOP
          FOR C_TEMP IN (SELECT CDPRODUTO, CDSUBPRODUTO, QTSUBPRODUTO, QTSUBPRODCALC, NRNIVEL, IDVERIFCOMP, NRORDEM, SGUNIDADE, NMPRODUTO, NMFANTPRODUTO
                           FROM TABLE(COMP_PROD_ORE(P_CDFILIAL, C_COMPLOCAL.CDSUBPRODLOC, C_COMPLOCAL.QTSUBPRODLOC, P_NRNIVEL + 1, P_ORIGEM))
                          ORDER BY NRORDEM) LOOP
              IF C_TEMP.CDPRODUTO = C_TEMP.CDSUBPRODUTO THEN
                 OUT_RECORD.CDPRODUTO := P_CDPRODUTO;
              ELSE
                 OUT_RECORD.CDPRODUTO  := C_TEMP.CDPRODUTO;
              END IF;
              OUT_RECORD.NRNIVEL       := C_TEMP.NRNIVEL;
              OUT_RECORD.CDSUBPRODUTO  := C_TEMP.CDSUBPRODUTO;
              OUT_RECORD.QTSUBPRODUTO  := C_TEMP.QTSUBPRODUTO;
              OUT_RECORD.QTSUBPRODCALC := C_TEMP.QTSUBPRODCALC * P_QUANTIDADE;
              OUT_RECORD.IDVERIFCOMP   := C_TEMP.IDVERIFCOMP;
              OUT_RECORD.SGUNIDADE     := C_COMPLOCAL.SGUNIDADE;
              OUT_RECORD.NMPRODUTO     := C_COMPLOCAL.NMPRODUTO;
              OUT_RECORD.NMFANTPRODUTO :=C_COMPLOCAL.NMFANTPRODUTO;
              OUT_RECORD.NRORDEM       := C_TEMP.NRORDEM;
              OUT_RECORD.NRORDEM       := C_COMPLOCAL.NRSEQPRODLOC;
              PIPE ROW (OUT_RECORD);
          END LOOP;
      END LOOP;
      
      SELECT NMPRODUTO, SGUNIDADE, NMFANTPRODUTO 
        INTO OUT_RECORD.NMPRODUTO, OUT_RECORD.SGUNIDADE, OUT_RECORD.NMFANTPRODUTO 
        FROM PRODUTO 
       WHERE CDPRODUTO = P_CDPRODUTO
       ORDER BY NRORG; 
      
      OUT_RECORD.CDPRODUTO     := P_CDPRODUTO;
      OUT_RECORD.CDSUBPRODUTO  := P_CDPRODUTO;
      OUT_RECORD.NRNIVEL       := P_NRNIVEL;
      OUT_RECORD.QTSUBPRODUTO  := P_QUANTIDADE;
      OUT_RECORD.QTSUBPRODCALC := P_QUANTIDADE;      
      OUT_RECORD.IDVERIFCOMP   := 'S';
      PIPE ROW (OUT_RECORD);
   ELSE
      SELECT COUNT(*)
        INTO V_COTPADR
        FROM ROTFABRICA R
       WHERE (R.CDPRODUTO = P_CDPRODUTO);
      IF V_COTPADR > 0 THEN
          /* Composio padro */
         FOR C_COMPPADR IN (SELECT R.CDPRODUTO, R.CDSUBPRODUTO, R.QTSUBPRODROT, R.NRSEQPROD, P.SGUNIDADE, P.NMPRODUTO, P.NMFANTPRODUTO
                              FROM ROTFABRICA R, PRODUTO P
                             WHERE (R.CDPRODUTO = P_CDPRODUTO)
                               AND (R.CDSUBPRODUTO = P.CDPRODUTO)
                               AND (NVL(R.IDGERAPEDORCAEVE,'S') = CASE WHEN V_IDTPPRODPEDOREV = 'A' AND P_ORIGEM = 'PRODECOMP' 
                                                                       THEN 'S' 
                                                                       ELSE NVL(R.IDGERAPEDORCAEVE, 'S') END)
                             ORDER BY R.NRSEQPROD)LOOP
             FOR C_TEMP IN (SELECT CDPRODUTO, CDSUBPRODUTO, QTSUBPRODUTO, QTSUBPRODCALC, NRNIVEL, IDVERIFCOMP, NRORDEM, SGUNIDADE, NMPRODUTO, NMFANTPRODUTO
                              FROM TABLE(COMP_PROD_ORE(P_CDFILIAL, C_COMPPADR.CDSUBPRODUTO, C_COMPPADR.QTSUBPRODROT, P_NRNIVEL + 1, P_ORIGEM))
                             ORDER BY NRORDEM) LOOP
                 IF C_TEMP.CDPRODUTO = C_TEMP.CDSUBPRODUTO THEN
                    OUT_RECORD.CDPRODUTO  := P_CDPRODUTO;
                 ELSE
                    OUT_RECORD.CDPRODUTO  := C_TEMP.CDPRODUTO;
                 END IF;
                 OUT_RECORD.CDSUBPRODUTO  := C_TEMP.CDSUBPRODUTO;
                 OUT_RECORD.NRNIVEL       := C_TEMP.NRNIVEL;
                 OUT_RECORD.QTSUBPRODUTO  := C_TEMP.QTSUBPRODUTO;
                 OUT_RECORD.QTSUBPRODCALC := C_TEMP.QTSUBPRODCALC * P_QUANTIDADE;
                 OUT_RECORD.SGUNIDADE     := C_TEMP.SGUNIDADE;
                 OUT_RECORD.NMPRODUTO     := C_TEMP.NMPRODUTO;
                 OUT_RECORD.NMFANTPRODUTO := C_TEMP.NMFANTPRODUTO;
                 OUT_RECORD.IDVERIFCOMP   := C_TEMP.IDVERIFCOMP;
                 OUT_RECORD.NRORDEM       := C_COMPPADR.NRSEQPROD;
                 PIPE ROW (OUT_RECORD);
             END LOOP;
         END LOOP;
         
         SELECT NMPRODUTO, SGUNIDADE, NMFANTPRODUTO
           INTO OUT_RECORD.NMPRODUTO, OUT_RECORD.SGUNIDADE, OUT_RECORD.NMFANTPRODUTO 
           FROM PRODUTO 
          WHERE CDPRODUTO = P_CDPRODUTO
          ORDER BY NRORG;
         
         OUT_RECORD.CDPRODUTO     := P_CDPRODUTO;
         OUT_RECORD.CDSUBPRODUTO  := P_CDPRODUTO;
         OUT_RECORD.NRNIVEL       := P_NRNIVEL;
         OUT_RECORD.QTSUBPRODUTO  := P_QUANTIDADE;
         OUT_RECORD.QTSUBPRODCALC := P_QUANTIDADE;
         OUT_RECORD.IDVERIFCOMP   := 'S';
         PIPE ROW (OUT_RECORD);
      ELSE      
         SELECT NMPRODUTO, SGUNIDADE, NMFANTPRODUTO
           INTO OUT_RECORD.NMPRODUTO, OUT_RECORD.SGUNIDADE, OUT_RECORD.NMFANTPRODUTO 
           FROM PRODUTO 
          WHERE CDPRODUTO = P_CDPRODUTO
          ORDER BY NRORG;         
        
         OUT_RECORD.CDPRODUTO     := P_CDPRODUTO;
         OUT_RECORD.CDSUBPRODUTO  := P_CDPRODUTO;
         OUT_RECORD.NRNIVEL       := P_NRNIVEL;
         OUT_RECORD.QTSUBPRODUTO  := P_QUANTIDADE;
         OUT_RECORD.QTSUBPRODCALC := P_QUANTIDADE;
         OUT_RECORD.IDVERIFCOMP   := 'N';
         PIPE ROW (OUT_RECORD);
      END IF;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('COMP_PROD_ORE','0.00.000'); END;
/
