var Fis00200_DocumentoDeArrecadacao_Controller = function(ScreenService, templateManager, Messages_fiscal, General, Fis00200_DocumentoDeArrecadacao_Service, ZhFilterPreferences) {

    var self = this;

    this.service = Fis00200_DocumentoDeArrecadacao_Service;

    var showGuiaRj = false; //usada nas funções verificaEstadoEmissao e salvarEdicaoGuia

    // Variável que armazenam os valores inseridos do filtro.
    var filteredData = {};

    /*#############################################################
    #                                                             #
    #                   Funções do Filtro                         #
    #                                                             #
    #############################################################*/

    /*
        Abertura do filtro da tela de Documento de Arrecação com o padrão
        novo de implementação.
    */
    this.abreFiltroDocArrecadacao = function(widget) {
        ZhFilterPreferences.setOriginalFilter(widget.id);
        ZhFilterPreferences.openFilterWidget(widget);
    };

    /*
        Caso o estado da emissão da guia seja o RJ, existe um processo de 
        tranimissão e consulta. Então realizar o controle da tela para
        exibit os elementos necessários.
    */
    this.verificaEstadoEmissao = function() {
        var widget = templateManager.container.getWidget('widgetDocumentos');

        showGuiaRj = filteredData.IDORGARRECADNFE == 'E' && (filteredData.OPCGUIA == 'G' ? filteredData.UFFAVORECIDO : filteredData.SGESTADO) == 'RJ';

        var receita = widget.getField('CDRECEITA');
        var natureza = widget.getField('CDNATUREZA');

        /*
            IMPORTANTE - POR ENQUANTO A TRANSMISSAO ESTÁ SENDO APENAS POR UMA LINHA
            POIS NO RETORNO DO XML ESTÁ RETORNANDO SOMENTE UM CÓDIGO DE BARRAS PARA TODAS
            GUIAS TRANSMITIDAS
        */
        //widget.getAction ('transmitirLote').isVisible     = showGuiaRj;
        //widget.showCheckbox                               = showGuiaRj;

        widget.getAction('consultarLote').isVisible = !!(showGuiaRj || (filteredData.IDORGARRECADNFE === 'E' && filteredData.OPCGUIA === 'G' && (!['RJ', 'SP', 'ES'].includes(filteredData.SGESTADO))));

        widget.getField('IDTRANSMISSAO').isVisible = showGuiaRj;
        widget.getField('IDTRANSMISSAO').showOnList = showGuiaRj;

        widget.getField('NRSEQLOTE').isVisible = showGuiaRj;
        widget.getField('NRSEQLOTE').showOnList = showGuiaRj;

        widget.getField('CDDETRECEITA').isVisible = !!(filteredData.IDORGARRECADNFE === 'E' && filteredData.OPCGUIA === 'G' && (!['RJ', 'SP', 'ES'].includes(filteredData.SGESTADO)));

        receita.isVisible = !showGuiaRj;
        natureza.isVisible = showGuiaRj;
        natureza.showOnList = showGuiaRj;

        if (showGuiaRj) {
            delete(receita.validations.required);
            natureza.validations.required = true;
            widget.getField('IDTRANSMISSAO').dataSource.data.forEach(function(field) {
                if (field.value === 'P') {
                    field.description = 'Erro no Processamento';
                }
            });
        }
        else {
            delete(natureza.validations.required);
            receita.validations.required = true;
            widget.getField('IDTRANSMISSAO').dataSource.data.forEach(function(field) {
                if (field.value === 'P') {
                    field.description = 'Invalidada pelo Portal';
                }
            });
        }

        widget.resizeGridColumns();
        widget.redraw(true);

    };

    /*
        Preparação do filtro da tela com a passagem de parametros necessárias
        para a execução do datasource da tela.
    */
    this.preparaFiltro = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        var widgetPagamento = templateManager.container.getWidget('tabbedDocumentosArrecação');
        var actionEmitirGnreTxt = widget.getAction('emitirGnreTxt');

        widget.dataSourceFilter = [{
                "name": "DTINIPERIODO",
                "operator": "=",
                "value": filterRow.DTPERIODO.start
            },
            {
                "name": "DTFIMPERIODO",
                "operator": "=",
                "value": filterRow.DTPERIODO.end
            },
            {
                "name": "CDEMPRESA",
                "operator": "=",
                "value": filterRow.CDEMPRESA
            },
            {
                "name": "CDIMPOSTO",
                "operator": "=",
                "value": filterRow.CDIMPOSTO
            }
        ];

        /*Ajusta o datasource, o datasourcefilter e os Checkbox de acordo com o tipo de emissão de guia*/
        if (filterRow.IDTIPOEMISSAO == 'D') {
            //Guia por documento 
            if (filterRow.OPCGUIA == '') { //TODO Verificar pelo tipo de imposto é mais claro e lógico que esta. Mudar quando possível.
                //Imposto Federal
                widget.dataSource.name = "fis_guias#/crud/guia_documentoArrecadDocFederal";
            }
            else {
                //GNRE ou Guia do Estado
                widget.dataSource.name = "fis_guias#/crud/guia_documentoArrecadDoc";
                widget.dataSourceFilter.push({
                    "name": "CDINSCRICAO",
                    "operator": "=",
                    "value": filterRow.IDORGARRECADNFE === 'F' ? 'T' : filterRow.IDORGARRECADNFE === 'E' ? filterRow.CDINSCESTA : filterRow.CDINSCMUNI
                });
                widget.dataSourceFilter.push({
                    "name": "SGESTADO",
                    "operator": "=",
                    "value": filterRow.UFFAVORECIDO
                });
                widget.dataSourceFilter.push({
                    "name": "P_TPGUIA",
                    "operator": "=",
                    "value": filterRow.OPCGUIA
                });
            }
            Fis00200_DocumentoDeArrecadacao_Service.ajustaCamposApuracaoOuDocumento('D', widget);
            widget.showCheckbox = true;
        }
        else {
            //Guia por apuração
            widget.dataSource.name = "fis_guias#/crud/guia_documentoArrecadacao";
            widget.dataSourceFilter.push({
                "name": "CDINSCRICAO",
                "operator": "=",
                "value": filterRow.IDORGARRECADNFE === 'F' ? 'T' : filterRow.IDORGARRECADNFE === 'E' ? filterRow.CDINSCESTA : filterRow.CDINSCMUNI
            });
            //Opção do GNRE, há UF do Favorecido
            if (filterRow.OPCGUIA == 'G') {
                widget.dataSourceFilter.push({
                    "name": "SGESTADO",
                    "operator": "=",
                    "value": filterRow.UFFAVORECIDO
                });
            }
            Fis00200_DocumentoDeArrecadacao_Service.ajustaCamposApuracaoOuDocumento('A', widget);
            widget.showCheckbox = (!['SP'].includes(filterRow.UFFAVORECIDO));
        }

        widgetPagamento.dataSourceFilter = [{
            "name": "CDEMPRESA",
            "operator": "=",
            "value": filterRow.CDEMPRESA
        }];

        widget.fields.forEach(function(field) {
            if ((field.name == 'DTPERIODO') || (field.name == 'IDTIPOEMISSAO'))
                field.filterProperties.isVisible = false;
        });

        this.setStripeData(filterRow);
        this.salvaDadosFiltro(filterRow);
        this.setaValoresDefaultPagamento();
        this.verificaEstadoEmissao();

        //Tratativa da action da GNRE TXT
        actionEmitirGnreTxt.isVisible = (filterRow.OPCGUIA == 'G' || (filterRow.OPCGUIA == 'E' && (filterRow.SGESTADO == 'RJ' || filterRow.SGESTADO == 'ES')));
        if (filterRow.SGESTADO == 'RJ' && filterRow.OPCGUIA == 'E') {
            actionEmitirGnreTxt.label = 'Emitir DARJ';
        }
        else if (filterRow.SGESTADO == 'RJ' && filterRow.OPCGUIA == 'G') {
            actionEmitirGnreTxt.label = 'Emitir GNRE RJ';
        }
        else if ((filterRow.SGESTADO == 'ES' && filterRow.OPCGUIA == 'E') || (filterRow.UFFAVORECIDO == 'ES' && filterRow.OPCGUIA == 'G')) {
            actionEmitirGnreTxt.label = 'Emitir DUA - ES';
        }
        else if (filterRow.UFFAVORECIDO == 'SP' && filterRow.OPCGUIA == 'G') {
            actionEmitirGnreTxt.label = 'Emitir GNRE TXT';
        }
        else {
            actionEmitirGnreTxt.label = 'Emitir GNRE';
        }

        widget.redraw(true);

    };

    /*
     * Função para alterar exibição e obrigatoriedade de campos no filtro,
     * conforme tipo do imposto(Orgão Arrecadador).
     */
    this.controlaCamposFiltro = function(widget) {
        var filterRow = widget.currentRow;
        var inscrestad = widget.getField('CDINSCESTA');
        var inscrmunicip = widget.getField('CDINSCMUNI');
        var orgaoArrecador = filterRow.IDORGARRECADNFE;

        if (widget.currentRow.CDIMPOSTO) {
            // Valida quais campos deverão ser exibidos no filtro para cada imposto
            switch (orgaoArrecador) {
                case "F":
                    setCampoInvisible(inscrestad);
                    setCampoInvisible(inscrmunicip);
                    break;
                case "E":
                    setCampoObrigatorio(inscrestad, false);
                    setCampoInvisible(inscrmunicip);
                    break;
                case "M":
                    setCampoObrigatorio(inscrmunicip, false);
                    setCampoInvisible(inscrestad);
                    break;
                default:
                    setCampoInvisible(inscrestad);
                    setCampoInvisible(inscrmunicip);
                    widget.currentRow.SGESTADO = '';
                    widget.currentRow.CDPAIS = '';
                    break;
            }
            this.checkInscricaoHabilitada(widget);
        }
        else {
            setCampoInvisible(inscrestad);
            setCampoInvisible(inscrmunicip);
            widget.currentRow.SGESTADO = '';
            widget.currentRow.CDPAIS = '';
        }
        //Se o tipo de guia for apuração o imposto simples nacional é permitido no campo
        widget.getField("DSIMPOSTONFE").dataSourceFilter[0].value = widget.currentRow.IDTIPOEMISSAO == 'A' ? 'S' : 'N';
        this.controlaCamposFiltroGnre(widget);
    };

    /*Altera a exibição dos campos e actions relacionados à GNRE*/
    this.controlaCamposFiltroGnre = function(widget) {
        var opc_idtipoemissao = widget.currentRow.IDTIPOEMISSAO;
        var opc_cdimposto = widget.currentRow.CDIMPOSTO;
        var opc_opcguia = widget.currentRow.OPCGUIA;
        var opcaoguia = widget.getField('OPCGUIA');
        var uffavorecido = widget.getField('UFFAVORECIDO');

        //Se o tipo de guia for 'documento' este oferece opção para GNER ou Guia do estado e se for ICMS Estadual ou ICMS ST Estadual
        if ( /*opc_idtipoemissao == 'D' && */ (opc_cdimposto == '00' || opc_cdimposto == '80')) {
            setCampoObrigatorio(opcaoguia, false);
            if (opc_opcguia == 'G') { //Guia por GNRE
                setCampoObrigatorio(uffavorecido, false);
            }
            else {
                setCampoInvisible(uffavorecido);
            }
        }
        else {
            setCampoInvisible(opcaoguia);
            setCampoInvisible(uffavorecido);
        }
    };

    /*
     * Função para controlar o filtro por empresa e a propriedade readOnly do
     * campo de inscrição habilitado no filtro.
     */
    this.checkInscricaoHabilitada = function(filterPopup) {
        var cdempresa = filterPopup.currentRow.CDEMPRESA;
        var cdinscestaField = filterPopup.getField("CDINSCESTA");
        var cdinscmuniField = filterPopup.getField("CDINSCMUNI");

        // Reseta os valores de inscrição do filtro.
        filterPopup.currentRow.CDINSCESTA = '';
        filterPopup.currentRow.CDINSCMUNI = '';
        filterPopup.currentRow.SGESTADO = '';
        filterPopup.currentRow.CDPAIS = '';

        // Filtro a ser setado no dataSourceFilter do field de inscrição
        // exibido.
        var filter = [{
            "name": "CDEMPRESA",
            "operator": "=",
            "value": cdempresa
        }];

        // Altera a obrigatoriedade dos campos e atualiza os filtros de
        // inscrição quando a empresa for alterada.
        if (cdinscestaField.isVisible) {
            setCampoObrigatorio(cdinscestaField, !cdempresa);
            cdinscestaField.dataSourceFilter = filter;
            cdinscestaField.reload();

        }

        if (cdinscmuniField.isVisible) {
            setCampoObrigatorio(cdinscmuniField, !cdempresa);
            cdinscmuniField.dataSourceFilter = filter;
            cdinscmuniField.reload();

        }
    };

    this.setStripeData = function(filterRow) {
        var stripeWidget = templateManager.container.getWidget("stripe");

        stripeWidget.currentRow = {};
        stripeWidget.currentRow.IDTIPOEMISSAO = filterRow.IDTIPOEMISSAO.toString() === 'A' ? 'Apuração' : 'Documento';
        stripeWidget.currentRow.DSIMPOSTONFE = filterRow.DSIMPOSTONFE.toString();
        stripeWidget.currentRow.DTPERIODO = 'De ' + filterRow.DTPERIODO.start + ' até ' + filterRow.DTPERIODO.end;
        stripeWidget.currentRow.DESCEMPRESA = filterRow.DESCEMPRESA.toString();
        stripeWidget.currentRow.CDINSCESTA = filterRow.CDINSCESTA.toString() + ' - ' + filterRow.SGESTADO.toString();
        stripeWidget.currentRow.CDINSCMUNI = filterRow.CDINSCMUNI.toString() + ' - ' + filterRow.SGESTADO.toString();

        stripeWidget.getField('CDINSCESTA').isVisible = (filterRow.IDORGARRECADNFE === 'E');
        stripeWidget.getField('CDINSCMUNI').isVisible = (filterRow.IDORGARRECADNFE === 'M');
    };

    /*
     * Função para armazenar os dados filtrados em variável dentro do escopo
     * deste controller. Essa variável é utilizada nos demais processos da
     * tela, facilitando-os.
     */
    this.salvaDadosFiltro = function(filterRow) {
        filteredData = Util.clone(filterRow);
    };

    this.controlaCampoImposto = function(args) {
        var widget = args.owner.field.widget;
        var imposto = widget.getField('DSIMPOSTONFE');
        widget.currentRow.CDIMPOSTO = '';
        widget.currentRow.IDORGARRECADNFE = '';
        imposto.clearValue(); //imposto.value('');
        this.controlaCamposFiltro(widget);
        /*imposto.reload().then(
                function(response) {
                    this.adicionaImpostoSimples(imposto,widget.currentRow.IDTIPOEMISSAO);
                }.bind(this)
        );*/
    };


    /*#############################################################
    #                                                             #
    #           Funções de Maninupalção de Elementos              #
    #                                                             #
    #############################################################*/



    /*
     * Função para esconder um campo, removendo quaisquer validações
     * associadas.
     */
    var setCampoInvisible = function(field) {
        field.isVisible = false;
        field.readOnly = true;
        field.validations = {};
        field.value('');
    };

    /*
     * Função para exibir um campo e setá-lo como obrigatório.
     */
    var setCampoObrigatorio = function(field, readOnly) {
        field.isVisible = true;
        field.readOnly = !!readOnly;
        field.validations = { required: true };
    };

    /*
        Função que ira controlar o Campo código de receita.
        Ele buscará informações estaduais caso o imposto seja estadual ou 
        federal caso o imposto seja federal.
   */
    this.controlaTipoReceita = function(widget, nomeWidget, nomeCampoGuia) {
        var widgetPopupReceita = templateManager.container.getWidget(nomeWidget).getField(nomeCampoGuia).selectWidget;
        var idtipoimp = filteredData.IDORGARRECADNFE;

        if (idtipoimp === 'E') {
            widgetPopupReceita.fields[0].showOnList = true;
            widgetPopupReceita.fields[1].showOnList = true;

            widgetPopupReceita.fields[2].showOnList = false;
            widgetPopupReceita.fields[3].showOnList = false;
            widgetPopupReceita.fields[4].showOnList = false;
            widgetPopupReceita.fields[5].showOnList = false;
            widgetPopupReceita.fields[6].showOnList = false;

            widgetPopupReceita.order = 'CDRECEITAEST';


        }
        else if (idtipoimp === 'F') {
            widgetPopupReceita.fields[0].showOnList = false;
            widgetPopupReceita.fields[1].showOnList = false;

            widgetPopupReceita.fields[2].showOnList = true;
            widgetPopupReceita.fields[3].showOnList = true;
            widgetPopupReceita.fields[4].showOnList = true;
            widgetPopupReceita.fields[5].showOnList = true;
            widgetPopupReceita.fields[6].showOnList = true;

            widgetPopupReceita.order = 'CDRECEITACONTR';

        }
        //widgetPopupReceita.redraw();
    };


    this.dataSourceTipoReceita = function(widget, nomeCampoReceita) {
        var campoReceita = widget.getField(nomeCampoReceita);
        var idtipoimp = filteredData.IDORGARRECADNFE;

        if (idtipoimp === 'E') {
            campoReceita.template = 'zh-select-autocomplete#field/select-autocomplete.html';
            campoReceita.events = [{
                "name": "FieldAfterSelectOpen",
                "code": "Fis00200_DocumentoDeArrecadacao_Controller.controlaTipoReceita(args.owner.field.widget,'widgetDocumentos','CDRECEITA')"
            }, {
                "name": "FieldOnClick",
                "code": "args.owner.field.initializeField()"
            }];
            campoReceita.dataSource.name = 'fis#/crud/codreceitaest';
            campoReceita.valueField = 'CDRECEITAEST';
            campoReceita.descriptionField = 'NMRECEITAEST';
            campoReceita.displayedFields = [
                "CDRECEITAEST",
                "NMRECEITAEST"
            ];
            campoReceita.searchableFields = [
                "CDRECEITAEST",
                "NMRECEITAEST"
            ];
            campoReceita.outData = {
                "CDRECEITAEST": nomeCampoReceita,
                "NMRECEITAEST": "NMRECEITAEST"
            };
            campoReceita.dataSourceFilter = [{
                    "name": "CDPAIS",
                    "operator": "=",
                    "value": filteredData.CDPAIS
                },
                {
                    "name": "SGESTADO",
                    "operator": "=",
                    "value": filteredData.SGESTADO
                }
            ];
            campoReceita.reload();
        }
        else if (idtipoimp === 'F') {
            campoReceita.template = 'zh-select-autocomplete#field/select-autocomplete.html';
            campoReceita.valueField = 'CDRECEITACONTR';
            campoReceita.descriptionField = 'NMRECEITACONTR';
            campoReceita.displayedFields = [
                "CDRECEITACONTR",
                "NMRECEITACONTR"
            ];
            campoReceita.searchableFields = [
                "CDRECEITACONTR",
                "NMRECEITACONTR"
            ];
            campoReceita.dataSource.name = 'fis#/crud/codreceitacontr';
            campoReceita.outData = {
                "CDRECEITACONTR": nomeCampoReceita,
                "NMRECEITACONTR": "NMRECEITACONTR",
                "CDVARIACAO": "CDVARIACAO",
                "IDPERIODICREC": "IDPERIODICREC",
                "IDDEBSCPINC": "IDDEBSCPINC"
            };
            campoReceita.dataSourceFilter = [{
                "name": "CDIMPOSTO",
                "operator": "=",
                "value": filteredData.CDIMPOSTO
            }];
            campoReceita.events = [{
                "name": "FieldAfterSelectOpen",
                "code": "Fis00200_DocumentoDeArrecadacao_Controller.controlaTipoReceita(args.owner.field.widget,'widgetDocumentos','CDRECEITA')"
            }, {
                "name": "FieldOnClick",
                "code": "args.owner.field.initializeField()"
            }];
            campoReceita.reload();
        }
        else {
            campoReceita.template = "field/text-edit.html";
            campoReceita.events = [];
        }

    };

    self.controlaDetalhamentoGnre = function(widget) {
        widget.getField('CDDETRECEITA').dataSourceFilter = [{
                "name": "CDRECEITA",
                "operator": "=",
                "value": widget.currentRow.CDRECEITA
            },
            {
                "name": "SGESTADO",
                "operator": "=",
                "value": filteredData.SGESTADO
            }
        ];

        widget.getField('CDDETRECEITA').reload();
    };

    /*
        Função que ira controlar a visibilade das actions na tela
        de acordo com o status da guia.
    */
    this.controlaActions = function(widget) {
        var idTipoEmissao = widget.appliedFilter.FILTER_ROW.IDTIPOEMISSAO;
        var editar = widget.getAction('EDITAR');
        var prepararPag = widget.getAction('prepararPagamento');
        var desfazerPrep = widget.getAction('desfazerPreparacao');
        var campoReceita = widget.getField('CDRECEITA');
        var imposto = widget.currentRow.CDIMPOSTO;

        var condicaoEditar;
        var condicaoPag;
        var condicaoDesf;

        if (idTipoEmissao == 'D') { //Emissão por documento
            condicaoEditar = true;
            condicaoPag = true;
            condicaoDesf = true;
        }
        else { //Emissão por apuração (lógica original)
            condicaoEditar = (widget.currentRow.IDBLOQMAPAPU === 'S' && widget.currentRow.IDSTATBPAG === 'P');
            condicaoPag = (widget.currentRow.IDBLOQMAPAPU === 'S' && widget.currentRow.IDSTATBPAG === 'P');
            condicaoDesf = (widget.currentRow.IDBLOQMAPAPU === 'S' && (widget.currentRow.IDSTATBPAG === 'B' || widget.currentRow.IDSTATBPAG === 'A'));
        }
        editar.readOnly = !condicaoEditar;
        prepararPag.readOnly = !condicaoPag;
        desfazerPrep.readOnly = !condicaoDesf;

        if (imposto == 'SN') { //Para o Simples Nacional o campo de código de receita não é obrigatório
            campoReceita.validations = {};
        }
        else {
            campoReceita.validations = { "required": true };
        }
    };

    /*this.adicionaImpostoSimples = function (field, tipoGuia){
        var data = field.dataSource.data;

        // Se a opção para emissão do documento for por mapa
        // adicionar a opção do simples nacional
        
        if(tipoGuia ==='A'){
            var simples  = {
                    "CDIMPOSTONFE":"SN",
                    "DSIMPOSTONFE":"Simples Nacional",
                    "IDORGARRECADNFE":null,
                    "SGIMPOSTONFE":"Simples Nacional"
                };
            //Verifica se Ja existe o simples nacional
            if(data.indexOf(simples) === -1)
                data.unshift(simples);
        }
    };*/


    /*#############################################################
    #                                                             #
    #                    Funções de Controle                      #
    #                                                             #
    #############################################################*/


    this.setaValoresDefaultPagamento = function() {
        var widgetPagamento = templateManager.container.getWidget('tabbedDocumentosArrecação');
        General.callRestEngine('fis_guias#/valoresDefaultPagamento', 'Row', filteredData).then(
            function(response) {
                var dadosPagamento = response.dataset.dadosPagamento;
                if (dadosPagamento.length > 0) {
                    widgetPagamento.currentRow.CDFILIAL = dadosPagamento[0].CDFILIAL;
                    widgetPagamento.currentRow.CDRATEIO = dadosPagamento[0].CDRATEIO;

                    widgetPagamento.getField('NMFILIAL').value(dadosPagamento[0].NMFILIAL);
                    widgetPagamento.getField('NMRATEIO').value(dadosPagamento[0].NMRATEIO);
                }
                else {
                    widgetPagamento.currentRow.CDFILIAL = null;
                    widgetPagamento.currentRow.CDRATEIO = null;

                    widgetPagamento.currentRow.NMFILIAL = null;
                    widgetPagamento.currentRow.NMRATEIO = null;
                }
                widgetPagamento.redraw(true);
            },
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.openPopupConsultaLote = function(widget) {
        var widgetConsulta = templateManager.container.getWidget('widgetConsultarLote');
        var lote = widgetConsulta.getField('NRSEQLOTE');
        ScreenService.openPopup(widgetConsulta);
        widgetConsulta.currentRow.NRSEQLOTE = '';
        widgetConsulta.currentRow.DTHRRECIBO = '';
        widgetConsulta.currentRow.IDSTATUSLOTE = '';

        lote.dataSourceFilter = [{
                "name": "CDEMPRESA",
                "operator": "=",
                "value": filteredData.CDEMPRESA
            },
            {
                "name": "CDINSCESTA",
                "operator": "=",
                "value": filteredData.CDINSCESTA
            },
            {
                "name": "SGESTADO",
                "operator": "=",
                "value": filteredData.SGESTADO
            }
        ];
        lote.reload();
    };



    /*#############################################################
    #                                                             #
    #                    Funções de Validações                    #
    #                                                             #
    #############################################################*/
    /*
     * Função chamada ao alterar o imposto selecionado. Verifica a
     * parametrização de impostos relacionados.
     */
    this.validaImpostosRelacionados = function(filterPopup) {
        var filterRow = filterPopup.currentRow;
        // Verifica se há um valor inserido para o campo de imposto.
        if (filterRow.CDIMPOSTO && filterRow.CDIMPOSTO !== 'SN') {
            General.callRestEngine('fis_guias#/vdImpostoRelac', 'Row', { "CDIMPOSTONFE": filterRow.CDIMPOSTO }).then(
                function(response) {
                    // Verifica se a validação retornou true ou false.
                    var impostoIsValid = response.dataset.validation.VDPARAMIMPRELAC;
                    if (!impostoIsValid) {
                        // Exibe a mensagem de erro e remove o imposto inserido.
                        ScreenService.showMessage(Messages_fiscal.PARAM_IMPOSTO_RELAC_ERROR, 'error');
                        filterRow.CDIMPOSTO = null;
                        filterRow.SGIMPOSTONFE = null;
                        filterRow.DSIMPOSTONFE = null;
                    }
                },
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }
    };


    this.verifcaStatusLote = function(lote) {
        var widget = lote.widget;
        if (widget.currentRow.IDSTATUSLOTE === 'S' || widget.currentRow.IDSTATUSLOTE === 'E') {
            ScreenService.showMessage(Messages_fiscal.LOTE_CONSULTADO);
            lote.value('');
            widget.currentRow.DTHRRECIBO = '';
            widget.currentRow.IDSTATUSLOTE = '';
        }

    };



    /*#############################################################
    #                                                             #
    #                    Funções de Processos                     #
    #                                                             #
    #############################################################*/


    this.salvarEdicaoGuia = function(widget) {
        var verificaRJ = self.verificaNaturezaRJ(widget);
        if (verificaRJ[0]) {
            if (widget.isValid()) {
                //Esta campo indica como a rotina deverá persistir os dados (para guia apuração e documento) e nas suas respectivas tabelas
                //widget.currentRow.IDTIPOEMISSAO = widget.appliedFilter.FILTER_ROW.IDTIPOEMISSAO;
                General.callRestEngine('fis_guias#/salvarEdicaoGuia', 'Row', Fis00200_DocumentoDeArrecadacao_Service.preparaRow(widget.currentRow)).then(
                    function(response) {
                        widget.computeDescriptionOrder();
                        widget.view();
                        ScreenService.closeSwipe();
                        widget.reload();
                        General.changeWidgetAcessibility(widget.widgets, true);
                    },
                    function(callback) {
                        if (callback.data) {
                            ScreenService.showMessage(callback.data.error);
                        }
                    }
                );
            }
        }
        else {
            ScreenService.showMessage(verificaRJ[1]);
            return;
        }
    };

    self.verificaNaturezaRJ = function(widget) {
        if (filteredData.SGESTADO == 'RJ') {
            if (filteredData.OPCGUIA == 'D') {
                if (filteredData.IDTIPOEMISSAO == 'D') {
                    if (!(['4', '5', '7', '58'].includes(widget.currentRow.CDNATUREZA))) return [false, 'Documento DARJ só pode receber as naturezas 4, 5, 7 ou 58'];
                }
                else if (filteredData.IDTIPOEMISSAO == 'A') {
                    if (!(['1', '2', '3', '4', '6', '56', '60', '61'].includes(widget.currentRow.CDNATUREZA))) return [false, 'Apuração DARJ só pode receber as naturezas 1, 2, 3, 4, 6, 56, 60 ou 61'];
                }
            }
            else if (filteredData.OPCGUIA == 'G') {
                if (filteredData.IDTIPOEMISSAO == 'D') {
                    if (!(['4', '5', '59'].includes(widget.currentRow.CDNATUREZA))) return [false, 'Documento GNRE só pode receber as naturezas 4, 5, ou 59'];
                }
                else if (filteredData.IDTIPOEMISSAO == 'A') {
                    if (!(['1', '2', '4', '57', '60'].includes(widget.currentRow.CDNATUREZA))) return [false, 'Apuração GNRE só pode receber as naturezas 1, 2, 4, 57 ou 60'];
                }
            }
        }
        return [true];

    };

    self.preparaPagamentoQuestion = function(widget) {
        var rows = widget.getCheckedRows();

        switch (self.verificaPreparaPagamento(rows)) {
            case 1:
                self.preparaPagamento(widget);
                break;

            case 2:
                ScreenService.confirmMessage(ScreenService.i18n('Essa preparação de pagamento não ira gerar um codigo de barras, por que a guia não foi enviada ou por sua consulta apresentar erros. Deseja continuar?'), 'question', function(success) {
                    self.preparaPagamento(widget);
                });
                break;

            case 0:
                ScreenService.showMessage('Não é possivel preparar pagamento de um lote não consultado. Consulte o lote antes de preparar o pagamento.');
                break;
        }

    };

    self.verificaPreparaPagamento = function(rows) {
        rows.forEach(row => {
            if (['N', 'P', 'U', 'C'].includes(row.IDTRANSMISSAO)) {
                return 2;
            }
            else if (row.IDTRANSMISSAO === 'L') {
                return 0;
            }
        });
        return 1;
    };

    self.preparaPagamento = function(widget) {

        var info = {};

        info.CDEMPRESA = filteredData.CDEMPRESA;
        info.CDIMPOSTO = filteredData.CDIMPOSTO;
        info.IDTIPOEMISSAO = filteredData.IDTIPOEMISSAO;
        info.CDMUNICIPIO = filteredData.CDMUNICIPIO;
        info.SGESTADO = filteredData.SGESTADO;
        info.IDTIPOEMISSAO = filteredData.IDTIPOEMISSAO;
        info.CDINSCESTA = filteredData.CDINSCESTA;
        info.CDINSCMUNI = filteredData.CDINSCMUNI;
        info.IDORGARRECADNFE = filteredData.IDORGARRECADNFE;
        info.DSIMPOSTONFE = filteredData.DSIMPOSTONFE;
        info.CDINSCRICAO = filteredData.IDORGARRECADNFE === 'M' ? filteredData.CDINSCESTA : (filteredData.IDORGARRECADNFE === 'E' ? filteredData.CDINSCESTA : 'T');

        var widgetPagamento = templateManager.container.getWidget('tabbedDocumentosArrecação').currentRow;

        var nrseqloteArray = self.removeDups(widget.getCheckedRows().map(row => row.NRSEQLOTE)).filter(function(el) {
            return el != null;
        });

        var dataset = [];
        if (nrseqloteArray.length > 0) {
            nrseqloteArray.forEach(NRSEQLOTE => {
                dataset.push(...widget.dataSource.data.filter(function(row) {
                    return NRSEQLOTE === row.NRSEQLOTE;
                }));
            });
        }

        var resul;
        if (dataset.length > 0) {
            dataset.forEach(function(row) {
                row.SGESTADO = filteredData.SGESTADO;
                resul = Fis00200_DocumentoDeArrecadacao_Service.validaPreparacaoPagamento(widgetPagamento, row);
                if (!resul) {
                    return;
                }
            });
        }

        if (resul === true) {
            var row = {
                "row": Fis00200_DocumentoDeArrecadacao_Service.preparaRow(dataset),
                "dadosPagamento": widgetPagamento,
                "info": info
            };

            General.callRestEngine('fis_guias#/preparaPagamento', 'Row', row).then(
                function(response) {

                    ScreenService.closeSwipe();
                    ScreenService.closePopup();
                    widget.reload().then(
                        function(response) {
                            this.controlaActions(widget);
                        }.bind(this)
                    );
                    if (response.dataset.response[0].length === 0) {
                        ScreenService.showMessage(Messages_fiscal.SUCESSO_PREPARACAO);
                    }
                    else {
                        var responseToString = Object.values(response.dataset.response).map(nrseqlote => '<br />' + nrseqlote).join(' ');
                        ScreenService.showMessage('Não foi realizado a preparação de pagamento para os seguintes lotes:' + responseToString);
                    }
                }.bind(this),
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }
        else
            ScreenService.showMessage(resul);

    };

    this.cancelaDadosFinanceiro = function(widget) {

        var info = {};
        info.CDINSCRICAO = filteredData.IDORGARRECADNFE === 'M' ? filteredData.CDINSCESTA : (filteredData.IDORGARRECADNFE === 'E' ? filteredData.CDINSCESTA : 'T');
        info.CDIMPOSTO = filteredData.CDIMPOSTO;
        info.CDEMPRESA = filteredData.CDEMPRESA;

        var nrseqloteArray = self.removeDups(widget.getCheckedRows().map(row => row.NRSEQLOTE)).filter(function(el) {
            return el != null;
        });

        var dataset = [];
        if (nrseqloteArray.length > 0) {
            nrseqloteArray.forEach(NRSEQLOTE => {
                dataset.push(...widget.dataSource.data.filter(function(row) {
                    return NRSEQLOTE === row.NRSEQLOTE;
                }));
            });
        }

        var row = {
            "row": Fis00200_DocumentoDeArrecadacao_Service.preparaRow(dataset),
            "info": info
        };
        General.callRestEngine('fis_guias#/cancelaDadosFinanceiro', 'Row', row).then(
            function(response) {
                ScreenService.closeSwipe();
                ScreenService.closePopup();
                widget.reload().then(
                    function(response) {
                        this.controlaActions(widget);
                    }.bind(this)
                );
                if (response.dataset.response[0].length === 0) {
                    ScreenService.showMessage(Messages_fiscal.SUCESSO_CANCELAMENTO);
                }
                else {
                    var responseToString = Object.values(response.dataset.response).map(nrseqlote => '<br />' + nrseqlote).join(' ');
                    ScreenService.showMessage('Não foi possivel desfazer a preparação de pagamento para os seguintes lotes:' + responseToString);
                }

            }.bind(this),
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.transmitirLote = function(args) {
        self.emitirGnreTxt(args, false);
    };

    self.removeDups = function(names) {
        var unique = {};
        names.forEach(function(i) {
            if (!unique[i]) {
                unique[i] = true;
            }
        });
        return Object.keys(unique);
    };

    self.transmitirLoteRj = function(widget) {
        var rows = widget.getCheckedRows();
        var infoRow = {};

        infoRow.CDEMPRESA = filteredData.CDEMPRESA;
        infoRow.CDMUNICIPIO = filteredData.CDMUNICIPIO;
        infoRow.SGESTADO = filteredData.SGESTADO;
        infoRow.IDTIPOEMISSAO = filteredData.IDTIPOEMISSAO;
        infoRow.TipoDocumento = filteredData.OPCGUIA == 'G' ? 2 : 1;
        infoRow.TipoApuracao = filteredData.IDTIPOEMISSAO == 'D' ? 2 : 1;
        infoRow.CDINSCESTA = filteredData.CDINSCESTA;
        infoRow.CDINSCMUNI = filteredData.CDINSCMUNI;
        infoRow.IDORGARRECADNFE = filteredData.IDORGARRECADNFE;
        infoRow.DSIMPOSTONFE = filteredData.DSIMPOSTONFE;
        infoRow.CDINSCRICAO = filteredData.IDORGARRECADNFE === 'M' ? filteredData.CDINSCESTA : (filteredData.IDORGARRECADNFE === 'E' ? filteredData.CDINSCESTA : 'T');

        var dataLimtPagArray = self.removeDups(rows.map(function(row) {
            return row.DTLIMITEPAG;
        }));

        var dataset = [Fis00200_DocumentoDeArrecadacao_Service.preparaRow(rows), infoRow, dataLimtPagArray];

        var resul = Fis00200_DocumentoDeArrecadacao_Service.validaTransmissaoLoteRj(rows);

        if (resul === true) {
            General.callRestEngine('fis_guias#/transmitirLoteRj', 'DataSet', dataset).then(
                function(response) {
                    ScreenService.closeSwipe();
                    ScreenService.closePopup();
                    widget.reload().then(
                        function(response) {
                            this.controlaActions(widget);
                        }.bind(this)
                    );

                    var screenServiceMenssage = '';
                    response.messages[0].message.forEach(function(envio) {
                        screenServiceMenssage = screenServiceMenssage + ' ' + envio.mensagem + ' Lote: ' + envio.nrseqlote + '.<br /><br />';
                    });
                    ScreenService.showMessage(screenServiceMenssage);
                }.bind(this),
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }
        else
            ScreenService.showMessage(resul);
    };

    self.transmitirLoteEs = function(widget) {
        var rows = widget.getCheckedRows();
        var infoRow = {};

        infoRow.UFFAVORECIDO = filteredData.UFFAVORECIDO;
        infoRow.CDEMPRESA = filteredData.CDEMPRESA;
        infoRow.CDMUNICIPIO = filteredData.CDMUNICIPIO;
        infoRow.CDIMPOSTO = filteredData.CDIMPOSTO;
        infoRow.SGESTADO = filteredData.SGESTADO;
        infoRow.IDTIPOEMISSAO = filteredData.IDTIPOEMISSAO;
        infoRow.OPCGUIA = filteredData.OPCGUIA;
        infoRow.IDTIPOEMISSAO = filteredData.IDTIPOEMISSAO;
        infoRow.CDINSCESTA = filteredData.CDINSCESTA;
        infoRow.CDINSCMUNI = filteredData.CDINSCMUNI;
        infoRow.IDORGARRECADNFE = filteredData.IDORGARRECADNFE;
        infoRow.DSIMPOSTONFE = filteredData.DSIMPOSTONFE;
        infoRow.CDINSCRICAO = filteredData.IDORGARRECADNFE === 'M' ? filteredData.CDINSCESTA : (filteredData.IDORGARRECADNFE === 'E' ? filteredData.CDINSCESTA : 'T');

        var dataset = [Fis00200_DocumentoDeArrecadacao_Service.preparaRow(rows), infoRow];

        General.callRestEngine('fis_guias#/transmitirLoteEs', 'DataSet', dataset).then(
            function(response) {
                ScreenService.closeSwipe();
                ScreenService.closePopup();
                widget.reload().then(
                    function(response) {
                        this.controlaActions(widget);
                    }.bind(this)
                );

                var screenServiceMenssage = '';
                response.messages[0].message.forEach(function(envio) {
                    screenServiceMenssage = screenServiceMenssage + ' ' + envio.mensagem + ' Lote: ' + envio.nrseqlote + '.<br /><br />';
                });
                ScreenService.showMessage(screenServiceMenssage);
            }.bind(this),
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );

    };

    self.emitirGnreOnline = function(widget) {
        var rows = widget.getCheckedRows();
        var infoRow = {};

        infoRow.CDEMPRESA = filteredData.CDEMPRESA;
        infoRow.CDMUNICIPIO = filteredData.CDMUNICIPIO;
        infoRow.SGESTADO = filteredData.SGESTADO;
        infoRow.CDIMPOSTO = filteredData.CDIMPOSTO;
        infoRow.UFFAVORECIDO = filteredData.UFFAVORECIDO;
        infoRow.IDTIPOEMISSAO = filteredData.IDTIPOEMISSAO;
        infoRow.TipoDocumento = filteredData.OPCGUIA == 'G' ? 2 : 1;
        infoRow.TipoApuracao = filteredData.IDTIPOEMISSAO == 'D' ? 2 : 1;
        infoRow.CDINSCESTA = filteredData.CDINSCESTA;
        infoRow.CDINSCMUNI = filteredData.CDINSCMUNI;
        infoRow.IDORGARRECADNFE = filteredData.IDORGARRECADNFE;
        infoRow.DSIMPOSTONFE = filteredData.DSIMPOSTONFE;
        infoRow.CDINSCRICAO = filteredData.IDORGARRECADNFE === 'M' ? filteredData.CDINSCESTA : (filteredData.IDORGARRECADNFE === 'E' ? filteredData.CDINSCESTA : 'T');


        var dataset = [Fis00200_DocumentoDeArrecadacao_Service.preparaRow(rows), infoRow];

        General.callRestEngine('fis_guias#/emitirGnreOnline', 'DataSet', dataset).then(
            function(response) {
                ScreenService.closeSwipe();
                ScreenService.closePopup();
                widget.reload().then(
                    function(response) {
                        this.controlaActions(widget);
                    }.bind(this)
                );

                var screenServiceMenssage = '';
                response.messages[0].message.forEach(function(envio) {
                    screenServiceMenssage = envio.status ? (screenServiceMenssage + ' ' + envio.mensagem + ' <br />Lote: ' + envio.nrseqlote + '.<br /><br />') : (screenServiceMenssage + ' ' + envio.mensagem + '.<br /><br />');
                });
                ScreenService.showMessage(screenServiceMenssage);
            }.bind(this),
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    self.consultarLote = function(widget) {
        if (filteredData.IDORGARRECADNFE == 'E' && (filteredData.OPCGUIA == 'G' ? filteredData.UFFAVORECIDO : filteredData.SGESTADO) == 'RJ') {
            self.consultarLoteRj(widget);
        }
        if (filteredData.IDORGARRECADNFE == 'E' && (filteredData.OPCGUIA == 'G' && (!['RJ', 'SP', 'ES'].includes(filteredData.SGESTADO)))) {
            self.consultarLoteGnreOnline(widget);
        }
    };

    self.consultarLoteRj = function(widget) {
        if (widget.isValid()) {
            widget.currentRow.CDMUNICIPIO = filteredData.CDMUNICIPIO;
            widget.currentRow.SGESTADO = filteredData.SGESTADO;
            widget.currentRow.IDTIPOEMISSAO = filteredData.IDTIPOEMISSAO;
            widget.currentRow.CDINSCESTA = filteredData.CDINSCESTA;
            widget.currentRow.CDINSCMUNI = filteredData.CDINSCMUNI;
            widget.currentRow.IDORGARRECADNFE = filteredData.IDORGARRECADNFE;

            var nomeArquivo = this.service.nomeArquivoDatetime;

            General.callRestEngine('fis_guias#/consultarLoteRj', 'Row', Fis00200_DocumentoDeArrecadacao_Service.preparaRow(widget.currentRow)).then(
                function(response) {
                    var consulta = response.dataset.consulta;
                    ScreenService.closeSwipe();
                    ScreenService.closePopup();

                    ScreenService.showMessage(consulta.mensagem).then(function() {
                        if (consulta.xml !== false) {
                            var blob = new Blob([response.dataset.consulta.xml], { type: "application/xml" });
                            saveAs(blob, nomeArquivo('gnre_consulta', '_', 'xml'));
                        }
                    });
                    templateManager.container.getWidget('widgetDocumentos').reload();

                }.bind(this),
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }


    };

    self.consultarLoteGnreOnline = function(widget) {
        if (widget.isValid()) {
            widget.currentRow.CDMUNICIPIO = filteredData.CDMUNICIPIO;
            widget.currentRow.SGESTADO = filteredData.SGESTADO;
            widget.currentRow.IDTIPOEMISSAO = filteredData.IDTIPOEMISSAO;
            widget.currentRow.CDINSCESTA = filteredData.CDINSCESTA;
            widget.currentRow.CDINSCMUNI = filteredData.CDINSCMUNI;
            widget.currentRow.CDEMPRESA = filteredData.CDEMPRESA;
            widget.currentRow.IDORGARRECADNFE = filteredData.IDORGARRECADNFE;

            var nomeArquivo = this.service.nomeArquivoDatetime;

            General.callRestEngine('fis_guias#/consultarLoteGnreOnline', 'Row', Fis00200_DocumentoDeArrecadacao_Service.preparaRow(widget.currentRow)).then(
                function(response) {
                    var consulta = response.dataset.consulta;
                    ScreenService.closeSwipe();
                    ScreenService.closePopup();

                    ScreenService.showMessage(consulta.mensagem).then(function() {
                        if (consulta.xml !== false) {
                            var blob = new Blob([response.dataset.consulta.xml], { type: "application/xml" });
                            saveAs(blob, nomeArquivo('gnre_consulta', '_', 'xml'));
                        }
                    });
                    templateManager.container.getWidget('widgetDocumentos').reload();

                }.bind(this),
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }


    };

    /*#############################################################
    #                                                             #
    #                    Funções de Emissão de Guias              #
    #                                                             #
    #############################################################*/


    this.emitirGuia = function(widget) {
        var widgetGuia = templateManager.container.getWidget('widgetDocumentos');
        widget.currentRow.CDMUNICIPIO = filteredData.CDMUNICIPIO;
        widget.currentRow.SGESTADO = filteredData.SGESTADO;
        widget.currentRow.IDTIPOEMISSAO = filteredData.IDTIPOEMISSAO;
        widget.currentRow.CDINSCESTA = filteredData.CDINSCESTA;
        widget.currentRow.CDINSCMUNI = filteredData.CDINSCMUNI;
        widget.currentRow.IDORGARRECADNFE = filteredData.IDORGARRECADNFE;
        widget.currentRow.UFFAVORECIDO = filteredData.UFFAVORECIDO;
        widget.currentRow.OPCGUIA = filteredData.OPCGUIA;
        widget.currentRow.CDIMPOSTO = filteredData.CDIMPOSTO;

        var resul = this.service.validaEmiisaoGuia(widgetGuia.currentRow);

        var row = Util.clone(widget.currentRow, ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"]);

        if (resul === true) {
            widget.currentRow.DataHoraSubmissao = this.service.DataHoraFim(0, 0, 0);

            switch (widget.currentRow.IDTIPOEMISSAO) {
                case 'A': //Apuração
                    //Simples Nacional
                    if (widget.currentRow.CDIMPOSTO == 'SN') {
                        this.service.emitirGuia(widget, row, null, null, 'http://www8.receita.fazenda.gov.br/SIMPLESNACIONAL/aplicacoes.aspx?id=19');
                    }
                    else {
                        //Demais impostos
                        if (widget.currentRow.IDORGARRECADNFE == 'E') {
                            switch (widget.currentRow.SGESTADO) {
                                case 'AC':
                                    this.service.emitirGuia(widget, row, null, null, 'http://www.sefaznet.ac.gov.br/sefazonline/servlet/hpfdaemn');
                                    break; //Acre
                                case 'AL':
                                    this.service.emitirGuia(widget, row, null, null, 'http://apl03.sefaz.al.gov.br/darcb/');
                                    break; //Alagoas
                                case 'AP':
                                    this.service.emitirGuia(widget, row, null, null, 'http://www.sefaz.ap.gov.br/sate/');
                                    break; //Amapá
                                case 'AM':
                                    this.service.emitirGuia(widget, row, null, null, 'http://online.sefaz.am.gov.br/daravulso/darOpcao.asp');
                                    break; //Amazonas
                                case 'BA':
                                    this.service.emitirGuia(widget, row, null, null, 'https://sistemas.sefaz.ba.gov.br/sistemas/arasp/pagamento/modulos/dae/pagamento/dae_pagamento.aspx');
                                    break; //Bahia
                                case 'CE':
                                    this.service.emitirGuia(widget, row, null, null, 'https://servicos.sefaz.ce.gov.br/internet/dae/taxas/default.asp');
                                    break; //Ceará
                                case 'DF':
                                    this.service.emitirGuia(widget, row, null, null, 'http://www.fazenda.df.gov.br/area.cfm?id_area=1075');
                                    break; //Distrito Federal
                                case 'ES':
                                    this.service.emitirGuia(widget, row, null, null, 'http://e-dua.sefaz.es.gov.br/');
                                    break; //Espírito Santo
                                case 'GO':
                                    this.service.emitirGuia(widget, row, null, null, 'https://app.sefaz.go.gov.br/arr-www/view/entradaContribuinte.jsf');
                                    break; //Goiás
                                case 'MA':
                                    this.service.emitirGuia(widget, row, null, null, 'http://aplicacoes.ma.gov.br/dare/');
                                    break; //Maranhão
                                case 'MT':
                                    this.service.emitirGuia(widget, row, null, null, 'https://www.sefaz.mt.gov.br/arrecadacao/darlivre/menudarlivre');
                                    break; //Mato Grosso
                                case 'MS':
                                    this.service.emitirGuia(widget, row, Messages_fiscal.EMITIR_DAEMS, 'daems', 'https://servicos.efazenda.ms.gov.br/sgae/EmissaoDAEMSdeICMS');
                                    break; //Mato Grosso do Sul
                                case 'MG':
                                    this.service.emitirGuia(widget, row, null, null, 'https://www2.fazenda.mg.gov.br/arrecadacao/ctrl/ARRECADA/ARRECADA/DOCUMENTO_ARRECADACAO?ACAO=VISUALIZAR');
                                    break; //Minas Gerais
                                case 'PA':
                                    this.service.emitirGuia(widget, row, null, null, 'https://app.sefa.pa.gov.br/emissao-dae-avulso/#/inicio');
                                    break; //Pará
                                case 'PB':
                                    this.service.emitirGuia(widget, row, null, null, 'https://www4.receita.pb.gov.br/atf/seg/SEGf_AcessarFuncao.jsp?cdFuncao=ARR_305');
                                    break; //Paraíba
                                case 'PR':
                                    this.service.emitirGuia(widget, row, Messages_fiscal.EMITIR_GRPR, 'grpr', 'http://www.fazenda.pr.gov.br/modules/conteudo/conteudo.php?conteudo=271');
                                    break; //Paraná
                                case 'PE':
                                    this.service.emitirGuia(widget, row, null, null, 'http://efisco.sefaz.pe.gov.br/sfi_trb_gae/PRGerarDAE');
                                    break; //Pernambuco
                                case 'PI':
                                    this.service.emitirGuia(widget, row, null, null, 'https://webas.sefaz.pi.gov.br/darweb/');
                                    break; //Piauí
                                case 'RJ':
                                    this.service.emitirGuia(widget, row, null, 'darj', null);
                                    break; //Rio de Janeiro
                                case 'RN':
                                    this.service.emitirGuia(widget, row, null, null, 'https://uvt.set.rn.gov.br/#/home');
                                    break; //Rio Grande do Norte
                                case 'RS':
                                    this.service.emitirGuia(widget, row, null, null, 'https://www.sefaz.rs.gov.br/EmissorGA/SAR/EmissorGaIcms.aspx');
                                    break; //Rio Grande do Sul
                                case 'RO':
                                    this.service.emitirGuia(widget, row, null, null, 'https://dareavulso.sefin.ro.gov.br/');
                                    break; //Rondônia
                                case 'RR':
                                    this.service.emitirGuia(widget, row, null, null, 'https://portalapp.sefaz.rr.gov.br/siate/servlet/wp_siate_dareavulso');
                                    break; //Roraima
                                case 'SC':
                                    this.service.emitirGuia(widget, row, Messages_fiscal.EMITIR_DARE_SC, 'dare-sc', 'https://tributario.sef.sc.gov.br/tax.NET/Sat.Arrecadacao.Web/DARE_online/EmissaoDareOnline.aspx');
                                    break; //Santa Catarina
                                case 'SP':
                                    this.service.emitirGuia(widget, row, Messages_fiscal.EMITIR_GARE, 'gare', 'https://www.fazenda.sp.gov.br/guiasinternet/gare/paginas/gare.aspx');
                                    break; //São Paulo
                                case 'SE':
                                    this.service.emitirGuia(widget, row, null, null, 'http://www.sefaz.se.gov.br/conteudo/267');
                                    break; //Sergipe
                                case 'TO':
                                    this.service.emitirGuia(widget, row, null, null, 'http://www.sefaz.to.gov.br/dare/servlet/hnetccwkda');
                                    break; //Tocantins
                                default:
                                    ScreenService.showMessage(Messages_fiscal.RELATORIO_NAO_DISPONIVEL);
                            } //fim do switch SGESTADO
                        }
                        else {
                            //Emissão da guia via sistema da receita, caso a guia esteja vencida este encaminha para o site/birt
                            this.service.guiaDarfReceitaOuBirt(widgetGuia, widget, row, filteredData);
                        }
                    }
                    break;
                case 'D': //Documento
                    if (widget.currentRow.IDORGARRECADNFE == 'E') {
                        switch (widget.currentRow.OPCGUIA) {
                            case 'E': //Guia do Estado
                                switch (widget.currentRow.SGESTADO) {
                                    case 'AC':
                                        this.service.emitirGuia(widget, row, null, null, 'http://www.sefaznet.ac.gov.br/sefazonline/servlet/hpfdaemn');
                                        break; //Acre
                                    case 'AL':
                                        this.service.emitirGuia(widget, row, null, null, 'http://apl03.sefaz.al.gov.br/darcb/');
                                        break; //Alagoas
                                    case 'AP':
                                        this.service.emitirGuia(widget, row, null, null, 'http://www.sefaz.ap.gov.br/sate/');
                                        break; //Amapá
                                    case 'AM':
                                        this.service.emitirGuia(widget, row, null, null, 'http://online.sefaz.am.gov.br/daravulso/darOpcao.asp');
                                        break; //Amazonas
                                    case 'BA':
                                        this.service.emitirGuia(widget, row, null, null, 'https://sistemas.sefaz.ba.gov.br/sistemas/arasp/pagamento/modulos/dae/pagamento/dae_pagamento.aspx');
                                        break; //Bahia
                                    case 'CE':
                                        this.service.emitirGuia(widget, row, null, null, 'https://servicos.sefaz.ce.gov.br/internet/dae/taxas/default.asp');
                                        break; //Ceará
                                    case 'DF':
                                        this.service.emitirGuia(widget, row, null, null, 'http://www.fazenda.df.gov.br/area.cfm?id_area=1075');
                                        break; //Distrito Federal
                                    case 'ES':
                                        this.service.emitirGuia(widget, row, null, null, 'http://e-dua.sefaz.es.gov.br/');
                                        break; //Espírito Santo
                                    case 'GO':
                                        this.service.emitirGuia(widget, row, null, null, 'https://app.sefaz.go.gov.br/arr-www/view/entradaContribuinte.jsf');
                                        break; //Goiás
                                    case 'MA':
                                        this.service.emitirGuia(widget, row, null, null, 'http://aplicacoes.ma.gov.br/dare/');
                                        break; //Maranhão
                                    case 'MT':
                                        this.service.emitirGuia(widget, row, null, null, 'https://www.sefaz.mt.gov.br/arrecadacao/darlivre/menudarlivre');
                                        break; //Mato Grosso
                                        // case 'MS': this.service.emitirGuia( widget, row, Messages_fiscal.EMITIR_DAEMS,   'daems',   'https://servicos.efazenda.ms.gov.br/sgae/EmissaoDAEMSdeICMS'                                     ); break; //Mato Grosso do Sul
                                    case 'MS':
                                        this.service.emitirGuia(widget, row, null, null, 'https://servicos.efazenda.ms.gov.br/sgae/EmissaoDAEMSdeICMS');
                                        break; //Mato Grosso do Sul
                                    case 'MG':
                                        this.service.emitirGuia(widget, row, null, null, 'https://www2.fazenda.mg.gov.br/arrecadacao/ctrl/ARRECADA/ARRECADA/DOCUMENTO_ARRECADACAO?ACAO=VISUALIZAR');
                                        break; //Minas Gerais
                                    case 'PA':
                                        this.service.emitirGuia(widget, row, null, null, 'https://app.sefa.pa.gov.br/emissao-dae-avulso/#/inicio');
                                        break; //Pará
                                    case 'PB':
                                        this.service.emitirGuia(widget, row, null, null, 'https://www4.receita.pb.gov.br/atf/seg/SEGf_AcessarFuncao.jsp?cdFuncao=ARR_305');
                                        break; //Paraíba
                                        // case 'PR': this.service.emitirGuia( widget, row, Messages_fiscal.EMITIR_GRPR,    'grpr',    'http://www.fazenda.pr.gov.br/modules/conteudo/conteudo.php?conteudo=271'                         ); break; //Paraná
                                    case 'PR':
                                        this.service.emitirGuia(widget, row, null, null, 'http://www.fazenda.pr.gov.br/modules/conteudo/conteudo.php?conteudo=271');
                                        break; //Paraná
                                    case 'PE':
                                        this.service.emitirGuia(widget, row, null, null, 'http://efisco.sefaz.pe.gov.br/sfi_trb_gae/PRGerarDAE');
                                        break; //Pernambuco
                                    case 'PI':
                                        this.service.emitirGuia(widget, row, null, null, 'https://webas.sefaz.pi.gov.br/darweb/');
                                        break; //Piauí
                                        //case 'RJ': this.service.emitirGuia( widget, row, null,                           'darj',     null                                                                                             ); break; //Rio de Janeiro
                                    case 'RJ':
                                        this.service.emitirGuia(widget, row, null, null, 'http://www1.fazenda.rj.gov.br/projetoGCTBradesco/br/gov/rj/sef/gct/web/emitirdocumentoarrecadacao/DocumentoArrecadacaoController.jpf');
                                        break; //Rio de Janeiro
                                    case 'RN':
                                        this.service.emitirGuia(widget, row, null, null, 'https://uvt.set.rn.gov.br/#/home');
                                        break; //Rio Grande do Norte
                                    case 'RS':
                                        this.service.emitirGuia(widget, row, null, null, 'https://www.sefaz.rs.gov.br/EmissorGA/SAR/EmissorGaIcms.aspx');
                                        break; //Rio Grande do Sul
                                    case 'RO':
                                        this.service.emitirGuia(widget, row, null, null, 'https://dareavulso.sefin.ro.gov.br/');
                                        break; //Rondônia
                                    case 'RR':
                                        this.service.emitirGuia(widget, row, null, null, 'https://portalapp.sefaz.rr.gov.br/siate/servlet/wp_siate_dareavulso');
                                        break; //Roraima
                                        // case 'SC': this.service.emitirGuia( widget, row, Messages_fiscal.EMITIR_DARE_SC, 'dare-sc', 'https://tributario.sef.sc.gov.br/tax.NET/Sat.Arrecadacao.Web/DARE_online/EmissaoDareOnline.aspx' ); break; //Santa Catarina
                                    case 'SC':
                                        this.service.emitirGuia(widget, row, null, null, 'https://tributario.sef.sc.gov.br/tax.NET/Sat.Arrecadacao.Web/DARE_online/EmissaoDareOnline.aspx');
                                        break; //Santa Catarina
                                        // case 'SP': this.service.emitirGuia( widget, row, Messages_fiscal.EMITIR_GARE,    'gare',    'https://www.fazenda.sp.gov.br/guiasinternet/gare/paginas/gare.aspx'                              ); break; //São Paulo
                                    case 'SP':
                                        this.service.emitirGuia(widget, row, null, null, 'https://www.fazenda.sp.gov.br/guiasinternet/gare/paginas/gare.aspx');
                                        break; //São Paulo
                                    case 'SE':
                                        this.service.emitirGuia(widget, row, null, null, 'http://www.sefaz.se.gov.br/conteudo/267');
                                        break; //Sergipe
                                    case 'TO':
                                        this.service.emitirGuia(widget, row, null, null, 'http://www.sefaz.to.gov.br/dare/servlet/hnetccwkda');
                                        break; //Tocantins
                                    default:
                                        ScreenService.showMessage(Messages_fiscal.RELATORIO_NAO_DISPONIVEL);
                                } //fim do switch SGESTADO
                                break;
                            case 'G': //GNRE
                                switch (widget.currentRow.SGESTADO) {
                                    case 'SP':
                                        this.service.emitirGuia(widget, row, Messages_fiscal.EMITIR_GNRE_SP, 'gnre-sp', 'https://www.fazenda.sp.gov.br/guiasinternet/Gare/Paginas/Gare.aspx');
                                        break;
                                    default:
                                        ScreenService.showMessage(Messages_fiscal.RELATORIO_NAO_DISPONIVEL);
                                }
                                break;
                        } //fim do switch OPCGUIA
                    }
                    else {
                        //Emissão da guia via sistema da receita, caso a guia esteja vencida este encaminha para o site/birt
                        this.service.guiaDarfReceitaOuBirt(widgetGuia, widget, row, filteredData); //TODO VER CAMPOS!!
                    }
                    break;
                default:
                    ScreenService.showMessage(Messages_fiscal.OPCAO_INVALIDA_GUIA);
            } //fim do switch IDTIPOEMISSAO
        }
        else
            ScreenService.showMessage(resul);
    };

    this.emitirGnreTxt = function(args, checkedRows = true) {
        //Somente as rows selecionadas
        var rows = checkedRows ? args.owner.widget.getCheckedRows() : [args.owner.widget.currentRow];

        if (!checkedRows) args.owner.widget.dataSource.addCheckedRows(rows[0]);

        var estado = filteredData.SGESTADO;
        //O this.service não está visível no escopo do function do response abaixo. Isto resolve este problema
        var nomeArquivo = this.service.nomeArquivoDatetime;

        //Somente prossegue com a geração se as rows selecionadas estiverem preenchidas
        if (this.service.validaCamposGnreTxt(rows, estado)) {
            //Requisição da string do texto do arquivo
            if (filteredData.OPCGUIA === 'E') {
                if (estado === 'RJ') {
                    self.transmitirLoteRj(args.owner.widget);
                }
                else if (estado === 'ES') {
                    self.transmitirLoteEs(args.owner.widget);
                }
            }
            else if (filteredData.OPCGUIA === 'G') {
                if (estado === 'RJ') {
                    self.transmitirLoteRj(args.owner.widget);
                }
                else if (filteredData.UFFAVORECIDO === 'ES') {
                    self.transmitirLoteEs(args.owner.widget);
                }
                else if (filteredData.UFFAVORECIDO === 'SP') {
                    General.callRestEngine('fis_guias#/emitirGnreTxt', 'Row', this.service.preparaRow(rows)).then(
                        function(response) {
                            //Criação e download do arquivo texto proveniente da string retornada
                            var blob = new Blob([response.dataset.arquivo[0]], { type: "text/plain" });
                            saveAs(blob, nomeArquivo('gnre', '_', 'txt'));
                        },
                        function(callback) {
                            if (callback.data) {
                                ScreenService.showMessage(callback.data.error);
                            }
                        }
                    );
                }
                else if (!['RJ', 'SP' /*, 'ES'*/ ].includes(filteredData.UFFAVORECIDO)) {
                    // teste ES usando IE de PE
                    // self.transmitirLoteEs(args.owner.widget);
                    self.emitirGnreOnline(args.owner.widget);
                }
            }
        }
    };

    self.controlaWidgetErro = function(widget) {
        var widgetErros = templateManager.container.getWidget('widgetErroConsultaLote');
        if (['P', 'U', 'C'].includes(widget.currentRow.IDTRANSMISSAO)) {
            widgetErros.isVisible = true;
        }
        else {
            widgetErros.isVisible = false;
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Fis00200_DocumentoDeArrecadacao_Controller", Fis00200_DocumentoDeArrecadacao_Controller);
});
